<?php
/**
 * Administration form to configure FEA.
 */
function fea_settings_form() {
  $form = array();
  $permissions = fea_permission();

  $form['form_ids'] = array(
    '#title' => t('Form IDs'),
    '#type' => 'fieldset',
  );
  $form['form_ids']['fea__display_form_id'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display all form ids for users with the %perm permission.', array(
      '%perm' => $permissions['administer fea settings']['title'],
    )),
    '#default_value' => variable_get('fea__display_form_id', FALSE),
  );

  $ids = variable_get('fea__form_ids', array());
  if (isset($_GET['add'])) {
    $ids[] = $_GET['add'];
  }
  $form['form_ids']['fea__form_ids'] = array(
    '#type' => 'textarea',
    '#title' => t('Enable FEA on these form ids'),
    '#description' => t('Put each form id on a new line. No wildcards allowed.'),
    '#default_value' => implode("\n", $ids),
  );

  return system_settings_form($form);
}

/**
 * Form validate function to explode form ids.
 *
 * @todo: cleanup deleted form settings
 */
function fea_settings_form_validate(&$form, &$form_state) {
  $ids = array_filter(explode("\n", trim($form_state['values']['fea__form_ids'])));
  sort($ids);
  $form_state['values']['fea__form_ids'] = array_unique(array_map('trim', $ids));
}

/**
 * Administration form to configure field access.
 *
 * @todo: auto enable parent fieldsets
 */
function fea_access_form() {
  $form = array();
  $roles = user_roles();
  $form_config = variable_get('fea__form_config', array());
  $form_ids = variable_get('fea__form_ids', array());

  foreach ($form_config as $form_id => $config) {
    $form[$form_id] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($form_id),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    if (!empty($config)) {
      $form[$form_id][_fea_variable_name('fea__invert__' . $form_id)] = array(
        '#type' => 'checkbox',
        '#title' => t('Revoke access to selected elements'),
        '#description' => t('Uncheck this to revoke access to all <em>unselected</em> elements'),
        '#default_value' => variable_get(_fea_variable_name('fea__invert__' . $form_id), FALSE),
      );
      foreach ($config as $key => $element) {
        $form[$form_id][_fea_variable_name($key)] = array(
          '#title' => t('@tree (@type)', array(
            '@tree' => implode(' > ', $element['tree']),
            '@type' => $element['type'],
          )),
          '#type' => 'checkboxes',
          '#options' => drupal_map_assoc(array_values($roles)),
          '#default_value' => variable_get($key, array()),
        );
      }
    }
    else {
      $form[$form_id]['#description'] = t('No form elements indexed.');
    }
  }

  // Check if all forms are indexed.
  foreach ($form_ids as $form_id) {
    if (!isset($form[$form_id])) {
      drupal_set_message(t('Please visit the page with form id %form_id on it to be able to configure the elements here.', array(
        '%form_id' => $form_id,
      )), 'warning', FALSE);
    }
  }

  // No form ids configured?
  if (empty($form_ids)) {
    drupal_set_message(t('No configured form ids. Please add them on the !link page.', array(
      '!link' => l(t('settings'), 'admin/config/user-interface/fea/settings'),
    )), 'error', FALSE);
  }

  return system_settings_form($form);
}

/**
 * Form validate function to filter out empty values.
 */
function fea_access_form_validate($form, &$form_state) {
  foreach ($form_state['values'] as $key => $value) {
    foreach (variable_get('fea__form_config', array()) as $form_id => $elements) {
      if (strpos($key, 'fea__' . $form_id) === 0 && is_array($value)) {
        $form_state['values'][$key] = array_filter($value);
      }
    }
  }
}
